unit Alrep;
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Printers, DrawMeta;

type
  {tOrientation = (poPortrait,poLandscape);}
  tPapersize   = (alr_A4,alr_A5,alr_Legal,alr_Letter);
type
  tAlBandType = (alr_Title,alr_PageHeader,
                   alr_DetailHeader,alr_Detail,alr_DetailFooter,
                   alr_PageFooter,alr_ReportFooter, alr_NewPage,
                   alr_GroupHeader, alr_GroupFooter);
type
  tAlSysDataType = (alr_Time,alr_Date,alr_DateTime,
                    alr_PageNum,alr_Counter,alr_Count,alr_FormName);

type
  tAlRulerType = (alr_none,alr_cmH, alr_cmHV, alr_cmV);

type
  tClipMode = (cm_Clip,cm_Expand,cm_Overwrite);

type
  tAlControl = class(tCustomLabel)
  private
    function BasicName:string;
  protected
    procedure SetParent (AParent:tWinControl); override;
    procedure Print (var aRect:tRect); virtual;
    procedure PrintOut (var aRect:tRect; const s:string); virtual; abstract;
    function  Skip:boolean; virtual;
    function  GetValue:string; virtual;
  public
    property  value:string read GetValue;
  published
    { verffentlichen geerbte Properties }
    property visible;
  end;

type
  TAlLabel = class(tAlControl)
  private
  protected
    Procedure AdjustPrintRect(VAR aRect:tRect);
    Procedure DoPrintText(var Rect:TRect; Flags: Word);
    Procedure PrintOut (var aRect:tRect; const s:string); override;
    procedure Print (var aRect:tRect); override;
  public
    constructor Create (AOwner:tComponent); override;
  published
    { verffentlichen geerbte Properties }
    property Caption;
    property Alignment;
    property AutoSize default true;
    property Transparent default true;
    property WordWrap default false;
    property Color;
    property Font;
    property ParentFont;
    property Enabled;
  end;

type
  TAlField = class(tAlLabel)
  private
    FValues : tStringList;
    dindex  : integer;
    procedure ResetData; virtual;
    procedure ClearData; virtual;
    function  Skip:boolean; override;
  protected
    function  GetValue:string; override;
    function  GetNextValue:string;
  public
    constructor Create (AOwner:tComponent); override;
    destructor  Destroy; override;
    property    NextValue:string read GetNextValue;
  published
    { verffentlichen geerbte Properties }
    property Caption;
    property Alignment;
    property AutoSize default true;
    property Transparent default true;
    property WordWrap default false;
    property Color;
    property Font;
    property ParentFont;
    property Enabled;
  end;

type
  TAlSysField = class(tAlLabel)
  private
    dIndex       : integer;
    FSysDataType : tAlSysDataType;
    procedure UpdateCaption;
  protected
    procedure Loaded; override;
    Procedure SetSysDataType (value:tAlSysDataType);
    function  Skip:boolean; override;
    function  GetValue:string; override;
  public
    { Public-Deklarationen }
    constructor Create (AOwner:TComponent); override;
  published
    { verffentlichen geerbte Properties }
    property Alignment;
    property AutoSize default true;
    property Transparent;
    property WordWrap;
    property Color;
    property Font;
    property ParentFont;
    property Enabled;
    { neue Properties }
    property DataType:tAlSysDataType read FSysDataType write SetSysDataType;
  end;

type
  TAlShape = class(tAlControl)
  private
    FBrush : tBrush;
    FPen   : tPen;
    FShape : tShapeType;
  protected
    procedure SetBrush(value:tBrush);
    procedure SetPen (value:tPen);
    procedure SetShape (value:tShapeType);
    procedure Paint; override;
    procedure PrintOut (var aRect:tRect; const dummy:string); override;
  public
    constructor Create (AOwner:tComponent); override;
    destructor  destroy; override;
  published
    procedure StyleChanged (Sender:tObject);
    { verffentlichen geerbte Properties }
    property Height default 65;
    property Width  default 65;
    { neue Properties }
    property Brush:tBrush read FBrush write SetBrush;
    property Pen:tPen read FPen write SetPen;
    property Shape:tShapeType read FShape write SetShape;
  end;

type
  tAlImage = class(tAlControl)
  private
    FPicture  : tPicture;
    FStretch  : boolean;
    FCenter   : boolean;
    aPen      : tPen;
  protected
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure Paint; override;
    procedure SetCenter (value:boolean);
    procedure SetStretch (value:boolean);
    procedure SetPicture (value:tPicture);
    procedure SetAutosize (value:boolean); override;
    procedure PrintOut (var aRect:tRect; const dummy:string); override;
  public
    constructor create (AOwner:tComponent); override;
    destructor  destroy; override;
  published
    procedure PicChanged (Sender:tObject);
    property Autosize;
    property Height   default 100;
    property Width    default 100;
    property Center:boolean read FCenter write SetCenter default true;
    property Stretch:boolean read FStretch write SetStretch default true;
    property Picture:tPicture read FPicture write SetPicture;
  end;

type
  tAlBand = class;
// type omitted for forward declaration
  TAlDetailLink = class(tComponent)
  private
    lastVal      : string;
    FGruppenKopf : tAlBand;
    FGruppenFuss : tAlBand;
    FDetailBand  : tAlBand;
    FDataSource  : TAlField;
    procedure SetDetailBand (value:tAlBand);
  protected
    Function CheckChanged (check_for:boolean):boolean;
  public
  published
    property GroupHeader:tAlBand read FGruppenKopf write FGruppenKopf;
    property GroupFooter:tAlBand read FGruppenFuss write FGruppenFuss;
    property GroupBand:tAlBand   read FDetailBand  write SetDetailBand;
    property DataSource:TAlField read FDataSource  write FDataSource;
  end;
// type omitted for forward declaration
  tAlBand = class(tCustomPanel)
  private
    FAbschnittTyp : tAlBandType;
    FAltColor     : boolean;
    alt           : boolean; // alternate the background color if not white
    FDetailLink   : tAlDetailLink;
    FRuler        : tAlRulerType;
    FClipMode     : tClipMode;
    procedure SetAbschnittTyp (value:tAlBandType);
    procedure SetRulerTyp (value:tAlRulerType);
  protected
    procedure SetParent (AParent:tWinControl); override;
    procedure Paint; override;
  public
    { Public-Deklarationen }
    constructor Create (AOwner:TComponent); override;
  published
    { verffentlichen geerbte Properties }
    property Align;
    property Color;
    property Font;
    property ParentFont;
    property Enabled;
    { neue Properties }
    property BandType:tAlBandType read FAbschnittTyp write SetAbschnittTyp;
    property AltColor:boolean read FAltColor write FAltColor default false;
    property Ruler:tAlRulerType read FRuler write SetRulerTyp;
    property ClipMode:tClipMode read FClipMode write FClipMode default cm_clip;
  end;

type
  tPages = class (tList)
     function GetPage(index:integer):tGraphic;
  public
     procedure Clear;
     property Page[index:integer]:tGraphic read GetPage;
  end;

type
  TAlReport = class(tComponent)
  private
    { Private-Deklarationen }
    RptHeaders   : tList;
    PgeHeaders   : tList;
    DtlHeaders   : tList;
    Details      : tList;
    DtlFooters   : tList;
    PgeFooters   : tList;
    RptFooters   : tList;
    YPos         : LongInt;
    MaxYPos      : LongInt;
    FirstPage    : boolean;

    { Property Functions }
    FOrientation : tPrinterOrientation;
    FPapersize   : tPapersize;
    FLeftMargin  : integer;
    FRightMargin : integer;
    FReportTitle : string;
    procedure SetOrientation (value:tPrinterOrientation);
    procedure SetPapersize   (value:tPapersize);
    procedure SetLeftMargin  (value:integer);
    procedure SetRightMargin (value:integer);
    procedure UpdateScrollBars;
    procedure PrintDtlHeader(nr:integer);
    procedure PrintDtlFooter(nr:integer);
    procedure PrintBand (aBand:tAlBand);
    procedure PrintBandNC (aBand:tAlBand);
    function  SkipBand (aBand:tAlBand):boolean;
    procedure FinishPage;
    procedure NewPage;
    Function  RealHeight(aHeight:LongInt):LongInt;
    Function  RealWidth(aWidth:longInt):LongInt;
    Function  LeftMargPix:LongInt;
    Function  RightMargPix:LongInt;
    procedure DoTheJob;
    procedure SetDebugMode (value:boolean);
    function  GetDebugMode:boolean;
    procedure SetCaptionMode (value:boolean);
    function  GetCaptionMode:boolean;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    constructor Create (AOwner:TComponent); override;
    destructor  destroy; override;
    procedure Print;
    procedure Preview(VAR ListOfPages:tPages);
    procedure AddNamedValue (const aName,aValue:string);
    procedure ResetData;
    procedure ClearData;
    property  DebugMode:boolean read GetDebugMode write SetDebugMode;
    property  CaptionMode:boolean read GetCaptionMode write SetCaptionMode;
  published
    { Published-Deklarationen }
    property Orientation:tPrinterOrientation read FOrientation write SetOrientation;
    property Papersize  :tPaperSize   read FPapersize   write SetPapersize;
    property LeftMarginMM:integer     read FLeftMargin  write SetleftMargin;
    property RightMarginMM:integer    read FRightMargin write SetRightMargin;
    property ReportTitle:string       read FReportTitle write FReportTitle;
  end;

type
  TAlPrinter = class // replacement for TPrinter enables previewing
  private
    FPreviewing              : boolean;
    FPages                   : tPages;  // list of metafiles
    FCurrentMetafile         : tDrawMetafile;
    FOrientation             : tPrinterOrientation;
    FLeftWaist               : integer; { in Pixel }
    FTopWaist                : integer; { in Pixel }
    FPhyPageWidth            : LongInt; { in Pixel }
    FPhyPageHeight           : LongInt;
    FPixelsPerInchVertical   : integer;
    FPixelsPerInchHorizontal : integer;
    FPaperSize               : tPaperSize;
    FTitle                   : string;
    function  GetCanvas:tCanvas;
    function  GetPageHeight:integer;
    function  GetPageWidth:integer;
    function  GetPageNumber:integer;
    function  GetPageCount:integer;
    procedure SetOrientation(value:tPrinterOrientation);
    procedure SetTitle (const s:string);
  protected
    property  Pages:tPages read FPages write FPages;
    procedure CalcMeasurements;
  public
    constructor Create; virtual;
    destructor  Destroy; virtual;
    procedure BeginDoc;
    procedure EndDoc;
    procedure NewPage;
    property  Canvas:tCanvas read GetCanvas;
    property  PageHeight:integer read GetPageHeight;
    property  PageWidth:integer read GetPageWidth;
    property  PageNumber:integer read GetPageNumber;
    property  Orientation:tPrinterOrientation read FOrientation write SetOrientation;
    property  Papersize:tPaperSize read FPapersize write FPapersize;
    property  Previewing:boolean read FPreviewing write FPreviewing;
    property  PageCount:integer read GetPageCount;
    property  LeftWaist:integer read FLeftWaist; { in Pixel }
    property  TopWaist:integer read FTopWaist; { in Pixel }
    property  PhyPageWidth:LongInt read FPhyPageWidth; { in Pixel }
    property  PhyPageHeight:LongInt read FPhyPageHeight;
    property  PixelsPerInchVertical:integer read FPixelsPerInchVertical;
    property  PixelsPerInchHorizontal:integer read FPixelsPerInchHorizontal;
    property  Title:string read FTitle write SetTitle;
  end;

  function AlPrinter:tAlPrinter;

  function LoadReport (const aFileName:string;
                       VAR aReport:tAlReport; VAR aForm:tForm):boolean;

  var
     NoValue : string;

  procedure Register;

implementation

const
   Inch   = 2.54;  // cm
   InchMM = 25.4;  // mm
const
   sizes : array[tPapersize] of tPoint =
    ((X:210; Y:297),(X:148;Y:210),(X:215;Y:355),(X:215;Y:279));

const
  BandName : array[tAlBandType] of string[16] =
     ('Title','PageHeader',
      'DetailHeader','Detail','DetailFooter',
      'PageFooter','ReportFooter','New Page',
      'GroupHeader','GroupFooter');

const
  check_group_header = false;
  check_group_footer = true;

const
  bDebugMode   : boolean = false;
  bCaptionMode : boolean = false;

type
   tBandList=class(tList)
      procedure SortInsert (aItem:tAlBand);
   end;

{---------------------------------------------------}

procedure tBandList.SortInsert (aItem:tAlBand);
var
  i : integer;
begin
  for i:=0 to Count-1 do
   if tAlBand(Items[i]).Top>aItem.Top then
    begin
      Insert (i,aItem);
      exit;
    end;
  Add (aItem);
end;

{---------------------------------------------------}
VAR
  FAlPrinter : tAlPrinter = NIL;

function AlPrinter:tAlPrinter;
begin
  if NOT Assigned(FAlPrinter) then
     FAlPrinter := tAlPrinter.Create;
  result := FAlPrinter;
end;

constructor tAlPrinter.Create;
begin
  inherited Create;
  FPreviewing := true;   // support for design mode
  CalcMeasurements;      // get design mode values

  FPages := tPages.Create;
end;

destructor tAlPrinter.Destroy;
var
  i : integer;
begin
  if Assigned(FCurrentMetafile) then
     FCurrentMetafile.Close;
  for i:=0 to FPages.Count-1 do
      with tDrawMetafile(FPages.Items[i]) do Free;
  FPages.Free;
  inherited Destroy;
end;

Procedure tAlPrinter.CalcMeasurements;
var
   FWaist      : tPoint;
   FPhySize    : tPoint;
   TextMetrics : TTextMetric;
begin
   if FPreviewing then
    begin
      FPixelsPerInchVertical := Screen.PixelsPerInch;
      FPixelsPerInchHorizontal := PixelsPerInchVertical;
      if FOrientation=poPortrait then
       begin
         FPhyPageWidth {pxl} := round (PixelsPerInchHorizontal
                                     * sizes[FPapersize].x / InchMM);
         FPhyPageHeight{pxl} := round (PixelsPerInchVertical
                                     * sizes[FPapersize].y / InchMM);
       end
      else
       begin
         FPhyPageWidth {pxl} := round (PixelsPerInchHorizontal
                                     * sizes[FPapersize].y / InchMM);
         FPhyPageHeight{pxl} := round (PixelsPerInchVertical
                                     * sizes[FPapersize].x / InchMM);
       end;
      FLeftWaist := 0;
      FTopWaist  := 0;
    end
   else
    begin
      // Calculate the number of pixels per inch vertical and horizontal.
      // 'GetDeviceCaps' is a Windows API call.
      FPixelsPerInchVertical   := GetDeviceCaps( Printer.Handle,LOGPIXELSY );
      FPixelsPerInchHorizontal := GetDeviceCaps( Printer.Handle,LOGPIXELSX );
      // Get the gutter on the left and top.  'Escape' is a Windows API call.
      Escape( Printer.Handle,GETPRINTINGOFFSET,0,Nil,@FWaist);
      FLeftWaist := FWaist.X;
      FTopWaist  := FWaist.Y;
      Escape( Printer.Handle,GETPHYSPAGESIZE,0,Nil,@FPhySize);
      FPhyPageWidth  := FPhySize.X;
      FPhyPageHeight := FPhySize.Y;
    end
end;

procedure tAlPrinter.BeginDoc;
var
  i : integer;
begin
  if FPreviewing then
     FPages.Clear
  else
     Printer.BeginDoc;
end;

procedure tAlPrinter.EndDoc;
begin
  if FPreviewing then
   begin
     if Assigned(FCurrentMetafile) then
        FCurrentMetafile.Close;
     FCurrentMetafile := NIL;
   end
  else
     Printer.EndDoc;
end;

procedure tAlPrinter.NewPage;
begin
  if FPreviewing then
   begin
     if Assigned(FCurrentMetafile) then
        FCurrentMetafile.Close;
     FCurrentMetafile := tDrawMetafile.Create (PhyPageWidth,PhyPageHeight);
     FPages.Add (FCurrentMetafile);
   end
  else
     Printer.NewPage;
end;

Function tAlPrinter.GetCanvas:tCanvas;
begin
  if FPreviewing then
   begin
     if NOT Assigned(FCurrentMetafile) then
        NewPage;
     if Assigned(FCurrentMetafile) then
        result := FCurrentMetafile.Canvas
     else
        raise Exception.Create ('AlPrinter could not assign the Canvas');
   end
  else
     result := Printer.Canvas;
end;

Function tAlPrinter.GetPageHeight:integer;
begin
  if FPreviewing then
     result := PhyPageHeight
  else
     result := Printer.PageHeight;
end;

function tAlPrinter.GetPageWidth:integer;
begin
  if FPreviewing then
     result := PhyPageWidth
  else
     result := Printer.PageWidth;
end;

Function tAlPrinter.GetPageNumber:integer;
begin
  if FPreviewing then
     result := FPages.Count
  else
     result := Printer.PageNumber;
end;

function tAlPrinter.GetPageCount:integer;
begin
  if FPreviewing then
     result := FPages.Count
  else
     result := 0;
end;

procedure tAlPrinter.SetOrientation(value:tPrinterOrientation);
begin
  FOrientation := value;
  if NOT FPreviewing then
     Printer.Orientation := FOrientation;
  CalcMeasurements;
end;

procedure tAlPrinter.SetTitle(const s:string);
begin
  FTitle := s;
  if NOT FPreviewing then
     Printer.Title := FTitle;
end;

///////////////////////////////////////////////////////////////////////////////

function tPages.GetPage(index:integer):TGraphic;
begin
  if (index>=0) and (index<Count) then
     result := tDrawMetafile(Items[index])
  else
     result := NIL;
end;

procedure tPages.Clear;
var
  i : integer;
begin
  for i:=0 to Count-1 do
     with tDrawMetafile(Items[i]) do Free;
  inherited Clear;
end;
///////////////////////////////////////////////////////////////////////////////

procedure TAlDetailLink.SetDetailBand (value:tAlBand);
begin
  if csDesigning in ComponentState then
     if Assigned(FDetailBand) then
        FDetailBand.FDetailLink := NIL;
     if Assigned(value) then
      begin
        FDetailBand := value;
        FDetailBand.FDetailLink := Self;
      end;
end;

function TAlDetailLink.CheckChanged (check_for:boolean):boolean;
begin
  result := false;
  if Assigned(FDataSource) then
   begin
     if check_for=check_group_footer then
      begin
        if Assigned(pChar(lastval)) then
           result := lastval<>FDataSource.NextValue
        else
           result := FALSE;
      end
     else // for header
      begin
        if Assigned(pChar(lastval)) then
           result := lastval<>FDataSource.value
        else
           result := TRUE;
        // get new value
        lastval := FDataSource.value;
      end;
   end;
end;

///////////////////////////////////////////////////////////////////////////////

constructor tAlReport.Create (AOwner:tComponent);
begin
   inherited Create (AOwner);
   FOrientation := poPortrait;
   FPapersize   := alr_A4;
   if csDesigning in ComponentState then
      UpdateScrollBars
   else
    with Owner as TForm do begin
      HorzScrollBar.Position := 0;
      VertScrollBar.Position := 0;
    end;
   bDebugMode := false;
   bCaptionMode := false;
end;

destructor tAlReport.destroy;
begin
   inherited destroy;
end;

procedure tAlReport.SetDebugMode(value:boolean);
begin
  bDebugMode := value;
end;

function tAlReport.GetDebugMode:boolean;
begin
  result := bDebugMode;
end;

procedure tAlReport.SetCaptionMode(value:boolean);
begin
  bCaptionMode := value;
end;

function tAlReport.GetCaptionMode:boolean;
begin
  result := bCaptionMode;
end;

procedure tAlReport.AddNamedValue (const aName,aValue:string);
var
  i       : integer;
  current : tAlField;
begin
   for i:=0 to Owner.ComponentCount-1 do
    if Owner.Components[i] is tAlField then
     begin
       current := tAlField(Owner.Components[i]);
       if UpperCase(current.Name)=UpperCase(aName) then
        begin
          current.FValues.Add(aValue);
          exit;
        end;
     end;
end;

procedure tAlReport.ResetData;
var
  i       : integer;
  current : tAlField;
begin
   for i:=0 to Owner.ComponentCount-1 do
    if Owner.Components[i] is tAlField then
     begin
       current := tAlField(Owner.Components[i]);
       current.ResetData;
     end;
end;

Procedure tAlReport.ClearData;
var
  current : tAlField;
  i       : integer;
begin
   for i:=0 to Owner.ComponentCount-1 do
    if Owner.Components[i] is tAlField then
     begin
       current := tAlField(Owner.Components[i]);
       current.ClearData;
     end;
end;

procedure tAlReport.UpdateScrollBars;
begin
   with tForm(Owner) do
    begin
      if FOrientation=poPortrait then
       begin
         HorzScrollBar.Range := Round(sizes[FPapersize].X
                             * Screen.PixelsPerInch/InchMM);
         VertScrollBar.Range := Round(sizes[FPapersize].Y
                             * Screen.PixelsPerInch/InchMM);
       end
      else
       begin
         VertScrollBar.Range := Round(sizes[FPapersize].X
                             * Screen.PixelsPerInch/InchMM);
         HorzScrollBar.Range := Round(sizes[FPapersize].Y
                             * Screen.PixelsPerInch/InchMM);
       end;
    end;
end;

procedure tAlReport.SetOrientation (value:tPrinterOrientation);
var
  i : integer;
begin
   if value<>FOrientation then
    begin
      FOrientation := value;
      AlPrinter.Orientation := value;

      if csDesigning in ComponentState then
       begin
         for i:=0 to Owner.ComponentCount-1 do
           if Owner.Components[i].ClassType=tAlBand then
              tAlBand(Owner.Components[i]).Refresh;
         UpdateScrollBars;
       end;
    end;
end;

procedure tAlReport.SetPapersize (value:tPapersize);
var
  i : integer;
begin
   if value<>FPapersize then
    begin
      FPapersize := value;
      AlPrinter.PaperSize := value;

      if csDesigning in ComponentState then
       begin
         for i:=0 to Owner.ComponentCount-1 do
           if Owner.Components[i].ClassType=tAlBand then
              tAlBand(Owner.Components[i]).Refresh;
         UpdateScrollBars;
       end;
    end;
end;

procedure tAlReport.SetLeftmargin(value:integer);
begin
   if (value>=0) and (value<Sizes[FPaperSize].X) then
      FLeftMargin := value;
end;

procedure tAlReport.SetRightmargin(value:integer);
begin
   if (value>=0) and (value<Sizes[FPaperSize].X) then
      FRightMargin := value;
end;

Procedure tAlReport.FinishPage;
var
  i : integer;
begin
  YPos := MaxYPos;
  for i:=0 to PgeFooters.Count-1 do
     PrintBandNC(PgeFooters.Items[i]);
end;

Procedure tAlReport.NewPage;
var
  i : integer;
begin
  if NOT FirstPage then
     AlPrinter.NewPage;
  FirstPage := False;
  YPos := 0;
  for i:=0 to PgeHeaders.Count-1 do
     PrintBandNC(PgeHeaders.Items[i]);
end;

Function tAlReport.RealHeight(aHeight:LongInt):LongInt;
var
  PaperHeight : integer;
begin
   // convert Screen Pixel <aHeight> to PrinterPixel <result>
   //
   if FOrientation=poPortrait then
      PaperHeight := Sizes[FPaperSize].Y
   else
      PaperHeight := Sizes[FPaperSize].X;

   result := Round(0.5 + aHeight * AlPrinter.PixelsPerInchVertical / Screen.PixelsPerInch);
end;

Function tAlReport.RealWidth(aWidth:LongInt):LongInt;
var
   PaperWidth : integer;
begin
   if FOrientation=poPortrait then
      PaperWidth := Sizes[FPaperSize].X
   else
      PaperWidth := Sizes[FPaperSize].Y;

  result := Round(0.5 + aWidth * AlPrinter.PixelsPerInchHorizontal / Screen.PixelsPerInch);
end;

Function tAlReport.LeftMargPix:LongInt;
var
   PaperWidth : integer;
begin
   if FOrientation=poPortrait then
      PaperWidth := Sizes[FPaperSize].X
   else
      PaperWidth := Sizes[FPaperSize].Y;
   // convert leftMargin in mm to Device Pixels
   result := Round(0.5 + leftMarginMM * AlPrinter.PixelsPerInchHorizontal / InchMM)
             - AlPrinter.LeftWaist;
end;

Function tAlReport.RightMargPix:LongInt;
var
   PaperWidth : integer;
begin
   if FOrientation=poPortrait then
      PaperWidth := Sizes[FPaperSize].X
   else
      PaperWidth := Sizes[FPaperSize].Y;
   // convert leftMargin in mm to Device Pixels
   result := Round(0.5 + RightMarginMM * AlPrinter.PixelsPerInchHorizontal / InchMM)
end;

procedure tAlReport.PrintBandNC (aBand:tAlBand);
var
  i  : integer;                 // iteration variable
  c  : tAlControl;              // Control to Print
  R  : tRect;                   // Band Rect
  cR : tRect;                   // Control Rect
  RR : tRect;                   // Result Rect
begin
  R := Rect(LeftMargPix+RealWidth(aBand.left),   yPos,
            RealWidth(aBand.Width)-RightMargPix, yPos+RealHeight(aBand.Height)-1);
  if aBand.AltColor then
   begin
     if aBand.Alt then
      begin
        AlPrinter.Canvas.Brush.Color := aBand.Color;
        AlPrinter.Canvas.FillRect (R);
      end;
     aBand.Alt := NOT aBand.Alt;
   end;

  for i:=0 to aBand.ControlCount-1 do
   begin
     c := tAlControl(aBand.Controls[i]);
     cR := Rect(RealWidth(c.left),
                RealHeight(c.top),
                RealWidth(c.left+c.width),
                RealHeight(c.top+c.height) );
     if c is tAlLabel then // affects: tAlLabel, tAlField, tAlSysfield
        tAlLabel(c).AdjustPrintRect (cR);
     cR.Left   := cR.Left + R.Left;
     cR.Right  := cR.Right + R.Left;
     cR.Top    := cR.Top   + R.Top;
     cR.Bottom := cR.Bottom + R.Top;
     case aBand.FClipMode of
       cm_Clip      : if IntersectRect(RR,R,cR) then // clip the Control Rect
                        c.Print (RR);
       cm_expand    : if UnionRect (RR,R,cR) then   // expand the Band Rect
                       begin
                         R := RR;
                         c.Print (cR);
                       end;
       cm_overwrite : c.Print (cR);
     end;
     // the Controls Print Rect
     // AlPrinter.Canvas.Brush.Style := bsClear;
     // AlPrinter.Canvas.Pen.Color   := clBlack;
     // AlPrinter.Canvas.Rectangle (cR.left,cr.top,cr.right,cr.bottom);
   end;
  // the Band Rect
  //AlPrinter.Canvas.Pen.Color   := clGreen;
  //AlPrinter.Canvas.Rectangle (R.left,r.top,r.right,r.bottom);
  YPos := 1 + R.Bottom; // YPos + RealHeight(aBand.Height);
end;

function tAlReport.SkipBand (aBand:tAlBand):boolean;
var
  more : boolean;
  c    : tAlControl;
  i    : integer;
begin
  result := false;
  for i:=0 to aBand.ControlCount-1 do
   begin
     c := tAlControl(aBand.Controls[i]);
     more := c.Skip;
     result := result OR more;
   end;
end;

procedure tAlReport.PrintDtlHeader(nr:integer);
var
  j : integer;
begin
  if nr<>0 then
     for j:=0 to DtlHeaders.Count-1 do
        if tAlBand(DtlHeaders.Items[j]).Tag=nr then
           PrintBand (DtlHeaders.Items[j]);
end;

procedure tAlReport.PrintDtlFooter(nr:integer);
var
  j : integer;
begin
  if nr<>0 then
     for j:=0 to DtlFooters.Count-1 do
        if tAlBand(DtlFooters.Items[j]).Tag=nr then
           PrintBand (DtlFooters.Items[j]);
end;

Procedure tAlReport.PrintBand (aBand:tAlBand);
var
  more   : boolean;
begin
  if aBand.FAbschnittTyp=alr_NewPage then
   begin
     FinishPage;
     // NewPage;
   end
  else
   begin
     repeat
       if YPos+RealHeight(aBand.Height)>MaxYPos then
        begin
             FinishPage;
             // conditionally print page headers
             NewPage;
             // if exist, print the detail header
             if aBand.FAbschnittTyp=alr_Detail then
              begin
                PrintDtlHeader (aBand.Tag);
                // when band is detail-linked then print group header
                if Assigned(aBand.FDetailLink) then
                 begin
                   with aBand.FDetailLink do
                    begin
                      CheckChanged (check_group_header);
                      if Assigned(FGruppenkopf) then
                         PrintBand (FGruppenKopf);
                    end;
                 end;
              end;
        end; // if YPos+RealHeight(aBand.Height)>MaxYPos then

       if Assigned(aBand.FDetailLink) then
        begin
          with aBand.FDetailLink do
             if CheckChanged (check_group_header) then
                if Assigned(FGruppenkopf) then
                   PrintBand (FGruppenKopf);
        end;

       PrintBandNC (aBand);

       if Assigned(aBand.FDetailLink) then
        begin
          with aBand.FDetailLink do
            if CheckChanged (check_group_footer) then
               if Assigned(FGruppenFuss) then
                  PrintBand (FGruppenFuss);
        end;

       if aBand.FAbschnittTyp=alr_Detail then
          more := SkipBand (aBand)
       else
          more := false;

     until (NOT more);
   end;
end;

procedure tAlReport.Print;
begin
  AlPrinter.Previewing := false;

  AlPrinter.Orientation := FOrientation;
  AlPrinter.Title := FReportTitle;
  AlPrinter.BeginDoc;
  AlPrinter.CalcMeasurements;

  ResetData;
  DoTheJob;
  AlPrinter.EndDoc;
end;

procedure tAlReport.Preview(VAR ListOfPages:tPages);
begin
  AlPrinter.Previewing := true;

  AlPrinter.Orientation := FOrientation;
  AlPrinter.CalcMeasurements;

  AlPrinter.Pages := ListOfPages;
  AlPrinter.BeginDoc;
  ResetData;
  DoTheJob;
  AlPrinter.EndDoc;
end;

Procedure tAlReport.DoTheJob;
var
   i,j      : integer;
   templist : tBandList;
   current  : tAlBand;
   lastab   : tAlBandType;
   lastTag  : integer;
begin
   { initialization }
   {----------------}
   { count and sort Abschnitte }
   templist := tBandList.Create;
   for i:=0 to Owner.ComponentCount-1 do
    if Owner.Components[i].ClassType=tAlBand then
     begin
       current := tAlBand(Owner.Components[i]);
       templist.SortInsert (current);
     end;

   RptHeaders := tBandList.Create;
   PgeHeaders := tBandList.Create;
   DtlHeaders := tBandList.Create;
   Details    := tBandList.Create;
   DtlFooters := tBandList.Create;
   PgeFooters := tBandList.Create;
   RptFooters := tBandList.Create;
   for i:=0 to templist.Count-1 do
    begin
      current := templist.Items[i];
      with current do
       begin
         case FAbschnittTyp of
           alr_Title       : RptHeaders.Add(current);
           alr_PageHeader  : PgeHeaders.Add(current);
           alr_DetailHeader : begin
                               current.Tag := i+1;
                               lastTag     := i+1;
                               DtlHeaders.Add(current);
                             end;
           alr_Detail      : begin
                               if lastab=alr_DetailHeader then
                                  current.Tag := lastTag
                               else
                                  lastTag := i+1;
                               Details.Add(current);
                             end;
           alr_newPage     : Details.Add(current);
           alr_DetailFooter : begin
                               if lastab=alr_Detail then
                                  current.Tag := lastTag;
                               DtlFooters.Add(current);
                             end;
           alr_PageFooter  : PgeFooters.Add(current);
           alr_ReportFooter        : RptFooters.Add(current);
         end;
         lastab := FAbschnittTyp;
       end;
    end;
   templist.free;

   { preparation }
   {-------------}
   MaxYPos := AlPrinter.PageHeight;
   for i:=0 to PgeFooters.Count-1 do
    begin
      current := PgeFooters.Items[i];
      MaxYPos := MaxYPos - RealHeight(current.Height);
    end;

   { printing }
   {----------}
   YPos := 99999;
   FirstPage := true;
   { print the Report-Header(s) }
   for i:=0 to RptHeaders.Count-1 do
      PrintBand (RptHeaders.Items[i]);
   { print Details }
   for i:=0 to Details.Count-1 do
    begin
      current := tAlBand(Details.Items[i]);
      { conditionaly print the Detail-Header(s) }
      PrintDtlHeader(current.Tag);
      { print the Detail }
      PrintBand (current);
      { conditionaly print the Detail-Footer(s) }
      PrintDtlFooter(current.Tag);
    end;
   { print the Report-Footer(s) }
   for i:=0 to RptFooters.Count-1 do
      PrintBand (RptFooters.Items[i]);
   FinishPage;

   { Cleanup }
   RptHeaders.Free;
   PgeHeaders.Free;
   DtlHeaders.Free;
   Details.Free;
   DtlFooters.Free;
   PgeFooters.Free;
   RptFooters.Free;
end;

///////////////////////////////////////////////////////////////////////////////

constructor tAlBand.Create (AOwner:tComponent);
begin
   inherited Create (AOwner);
   Color      := clWhite;
   BevelInner := bvNone;
   BevelOuter := bvNone;
   Ctl3D      := False;
   Caption    := '';
   Align      := alTop;
   FRuler     := alr_cmHV;

   FAbschnittTyp := alr_Title;
end;

procedure tAlBand.SetAbschnittTyp(value:tAlBandType);
begin
  if value<>FAbschnittTyp then
   begin
     FAbschnittTyp := value;
     Refresh;
   end;
end;

procedure tAlBand.SetRulerTyp(value:tAlRulerType);
begin
  if value<>FRuler then
   begin
     FRuler := value;
     Refresh;
   end;
end;
(*
procedure tAlBand.SetColumns(value:word);
begin
  if value<>FColumns then
   begin
     FColumns := value;
     Refresh;
   end;
end;
*)
procedure tAlBand.SetParent (AParent:tWinControl);
begin
  if (AParent<>NIL) and NOT (aParent is tForm) then
     raise Exception.Create ('This component must be placed on a tForm.')
  else
     inherited SetParent(AParent);
end;

procedure tAlBand.Paint;
var
   cm : integer;
   cw : integer;
begin
  inherited Paint;
  if (csDesigning in ComponentState) then
   begin
     Canvas.Font.Color := clSilver;
     Canvas.Font.Size  := 8;
     Canvas.Font.Name  := 'Arial';
     Canvas.Pen.Color  := clSilver;
     Canvas.Pen.Style  := psDash;
     if (FRuler<>alr_none) then
      begin
        if FRuler in [alr_cmHV,alr_cmV] then
           for cm:=1 to Round(Width*Inch/Screen.PixelsPerInch) do
            begin
              Canvas.MoveTo (Round(cm*Screen.PixelsPerInch/Inch),0);
              Canvas.LineTo (Round(cm*Screen.PixelsPerInch/Inch),Height);
              Canvas.TextOut(Round(cm*Screen.PixelsPerInch/Inch)+2,0,
                             IntToStr(cm)+'cm');
              Canvas.TextOut(0,10,BandName[FAbschnittTyp]);
            end;

        if FRuler in [alr_cmH, alr_cmHV] then
           for cm:=1 to Round(Height*Inch/Screen.PixelsPerInch) do
            begin
              Canvas.MoveTo (0,Round(cm*Screen.PixelsPerInch/Inch));
              Canvas.Lineto (Width,Round(cm*Screen.PixelsPerInch/Inch));
              Canvas.TextOut(0,Round(cm*Screen.PixelsPerInch/Inch)+2,
                             IntToStr(cm)+'cm');
            end;
      end; // if (FRuler<>alr_none) then
(*
     if FColumns>1 then
      begin
        Canvas.Pen.Style := psSolid;
        cw := AlPrinter.PhyPageWidth div FColumns;
        for cm:=1 to FColumns do
         begin
           Canvas.MoveTo (cm*cw,0);
           Canvas.LineTo (cm*cw,Height);
         end;
      end;
*)
   end; // if (csDesigning in ComponentState)
end;

///////////////////////////////////////////////////////////////////////////////

procedure tAlControl.SetParent(AParent:tWinControl);
begin
  if (AParent<>NIL) and NOT (aParent is tAlBand) then
     raise Exception.Create ('This component must be placed on a tAlBand.')
  else
     inherited SetParent(AParent);
end;

function tAlControl.Skip:boolean;
begin
  result := false;
end;

function tAlControl.GetValue:string;
begin
  result := Caption;
end;

procedure tAlControl.Print (var aRect:tRect);
begin
  if visible then
     PrintOut (aRect,value);
end;

function TAlControl.BasicName:string;
begin
  result := Name;
  while result[length(result)]='_' do // build basic name without underline
     result:= copy(result,1,length(result)-1);
end;

///////////////////////////////////////////////////////////////////////////////

constructor TAlLabel.Create (aOwner:tComponent);
begin
  inherited Create (aOwner);
  Autosize    := true;
  Transparent := true;
end;

procedure tAlLabel.AdjustPrintRect(Var aRect:tRect);
const
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  DC    : HDC;
  X     : Integer;
  cRect : tRect;
begin
  if Autosize then
   begin
     cRect := Rect(0,0,aRect.Right-aRect.Left,aRect.Bottom-aRect.Top);
     // DC := GetDC(0);
     // Canvas.Handle := DC;
     // NOW Print instead of Draw
     DoPrintText(cRect, (DT_EXPANDTABS or DT_CALCRECT) or WordWraps[WordWrap]);
     // Canvas.Handle := 0;
     // ReleaseDC(0, DC);
     X := aRect.Left;
     if Alignment = taRightJustify then Inc(X, (aRect.Right-aRect.Left) - cRect.Right);
     // SetBounds(X, Top, aRect.Right, aRect.Bottom);
     aRect.Left   := X;
     aRect.Right  := X + cRect.Right;
     aRect.Bottom := aRect.Top + cRect.Bottom;
   end;
end;

procedure tAlLabel.DoPrintText(var Rect:TRect; Flags: Word);
var
   Text : string;
begin
   Text := value;
   if (Flags and DT_CALCRECT <> 0) and (Text = '') then
      Text := Text + ' ';
   Flags := Flags or DT_NOPREFIX;
   AlPrinter.Canvas.Font := Self.Font;
   DrawText(AlPrinter.Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
end;

procedure tAlLabel.PrintOut (var aRect:tRect; const s:string);
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
begin
  if NOT Transparent then
   begin
     AlPrinter.Canvas.Brush.Color := Self.Color;
     AlPrinter.Canvas.Brush.Style := bsSolid;
     AlPrinter.Canvas.FillRect (aRect);
   end;
  AlPrinter.Canvas.Brush.Style := bsClear;
  DoPrintText(aRect, DT_EXPANDTABS or WordWraps[WordWrap] or
                   Alignments[Alignment]);
end;

procedure TAlLabel.Print (var aRect:tRect);
begin
  if visible then
     PrintOut (aRect,Value);
end;

///////////////////////////////////////////////////////////////////////////////

constructor TAlField.Create (aOwner:tComponent);
begin
  inherited Create (aOwner);  // inherits from tAlLabel
  if csDesigning in ComponentState then
    FValues := NIL
  else
   begin
     FValues := tStringList.Create;
     FValues.duplicates := dupAccept;
     dIndex := 0;
   end;
end;

destructor TAlField.Destroy;
begin
  FValues.Free;
  inherited Destroy;
end;

procedure tAlField.ResetData;
begin
  dIndex := 0;
end;

procedure tAlField.ClearData;
begin
  FValues.Clear;
  dIndex := 0;
end;

function tAlField.Skip:boolean;
begin
  result := false;
  if FValues.Count>1 then
   begin
     if dIndex<FValues.Count then
        inc (dIndex);
     result := dIndex<FValues.Count;
   end;
end;

function TAlField.GetValue:string;
var
  c : tComponent;
begin
  if FValues.Count=0 then
   begin
     if bCaptionMode then
        result := Caption
     else if bDebugMode then
        result := '['+Uppercase(Name)+']'
     else
     if Name[length(Name)]='_' then
      begin
        c := Owner.FindComponent(BasicName);
        if Assigned(c) then
           if c is tAlField then
              result := tAlField(c).Value
           else
              result := NoValue
        else
           result := NoValue;
      end
     else
        result := NoValue;
   end
  else if dIndex<FValues.Count then
     result := FValues.Strings[dIndex]
  else if FValues.Count=1 then
     result := FValues.Strings[0]
  else
     result := NoValue;
end;

function TAlField.GetNextValue:string;
var
  c : tComponent;
begin
  if FValues.Count=0 then
   begin
     if Name[length(Name)]='_' then
      begin
        c := Owner.FindComponent(BasicName);
        if Assigned(c) then
           if c is tAlField then
              result := tAlField(c).NextValue
           else
              result := Caption
        else
           result := Caption;
      end
     else
        result := Caption;
   end
  else if dIndex+1<FValues.Count then
     result := FValues.Strings[dIndex+1]
  else if FValues.Count=1 then
     result := FValues.Strings[0]
  else
     result := NoValue;
end;

///////////////////////////////////////////////////////////////////////////////

constructor TAlSysField.Create (AOwner:tComponent);
begin
   inherited Create (AOwner);
   Autosize    := true;
   Color       := clWhite;
   Transparent := true;

   DataType    := alr_Time;
end;

procedure TAlSysField.Loaded;
begin
  inherited Loaded;
  UpdateCaption;
end;

function TAlSysField.GetValue:string;
begin
   case FSysDataType of
     alr_Time     : result := TimetoStr(Time);
     alr_Date     : result := DateToStr(Date);
     alr_DateTime : result := DateTimeToStr(Now);
     alr_PageNum  : result := IntToStr(AlPrinter.PageNumber);
     alr_Counter  : result := IntToStr(1+dIndex);
     alr_Count    : result := 'E';
     alr_FormName : result := tForm(Owner).Name;
   end;
end;

function TAlSysField.Skip:boolean;
begin
  result := false;
  inc (dIndex);
end;

Procedure TAlSysField.SetSysDataType (value:tAlSysDataType);
begin
  // if value<>FSysDataType then
   begin
     FSysDataType := value;
     UpdateCaption;
   end;
end;

procedure TalSysField.UpdateCaption;
begin
  case FSysDataType of   // Caption is hidden, but inherited it's there !!
     alr_Time      : inherited Caption := '#hh:mm:ss#';
     alr_Date      : inherited Caption := '#tt.mm.jjjj#';
     alr_DateTime  : inherited Caption := '#tt.mm.jjjj hh:mm:ss#';
     alr_PageNum   : inherited Caption := '#Page#';
     alr_Counter   : inherited Caption := '#Counter#';
     alr_Count     : inherited Caption := '#Count#';
     alr_FormName  : inherited Caption := '#'+tForm(Owner).Name+'#';
  end;
end;

///////////////////////////////////////////////////////////////////////////////

constructor tAlShape.Create (AOwner:tcomponent);
begin
  inherited Create (AOwner);
  Caption := '';
  Autosize := false;
  height := 65;
  width  := 65;
  Transparent := TRUE;
  FShape := stRectangle;
  FBrush := tBrush.Create;
  FBrush.OnChange := StyleChanged;
  FPen   := tPen.Create;
  FPen.OnChange := StyleChanged;
end;

destructor tAlShape.Destroy;
begin
  FBrush.Free;
  FPen.Free;
  inherited destroy;
end;

procedure tAlShape.StyleChanged(Sender:tObject);
begin
  Refresh;
end;

procedure tAlShape.SetBrush (value:tBrush);
begin
  FBrush.Assign (value);
end;

procedure tAlShape.SetPen (value:tPen);
begin
  FPen.Assign (value);
end;

procedure tAlShape.SetShape (value:tShapeType);
begin
  if FShape<>value then
   begin
     FShape := value;
     Refresh;
   end;
end;

procedure tAlShape.PrintOut (var aRect:tRect; const dummy:string);
var
   ax,ay,aw,ah,s : Longint;
begin
  with AlPrinter.Canvas do
   begin
     Pen.Assign (FPen);
     if Pen.Width<1 then Pen.Width := 1;
     Brush.Assign (FBrush);
     aw := aRect.Right-aRect.Left;
     ah := aRect.Bottom-aRect.Top;
     if aw<ah then s := aw else s := ah;
     case FShape of
       stRectangle, stRoundRect, stEllipse :
        begin
          ax := aRect.Left+0;
          ay := aRect.Top+0;
        end;
       stSquare, stRoundSquare, stCircle :
        begin
          ax := aRect.Left+(aw-s) div 2;
          ay := aRect.Top+(ah-s) div 2;
          aw := s;
          ah := s;
        end;
     end;
     case FShape of
        stRectangle,
        stSquare      : if (aw<=1) or (ah<=1) then
                         begin
                           MoveTo (ax,ay);
                           if aw<=1 then LineTo (ax{+aw},ay+ah)
                                    else LineTo (ax+aw,ay{+ah});
                         end
                        else
                           Rectangle (ax,ay,ax+aw,ay+ah);
        stRoundRect,
        stRoundSquare : RoundRect (ax,ay,ax+aw,ay+ah,s div 4, s div 4);
        stEllipse,
        stCircle      : Ellipse (ax,ay,ax+aW,ay+ah);
     end;
   end; // with AlPrinter.Canvas do
end;

procedure tAlShape.Paint;
var
   x,y,w,h,s : integer;
begin
  with Canvas do
   begin
     Pen := FPen;
     Brush := FBrush;
     w := width;
     h := height;
     if w<h then s := w else s := h;
     case FShape of
       stRectangle, stRoundRect, stEllipse :
        begin
          x := 0;
          y := 0;
        end;
       stSquare, stRoundSquare, stCircle :
        begin
          x := (w-s) div 2;
          y := (h-s) div 2;
          w := s;
          h := s;
        end;
     end;
     case FShape of
        stRectangle,
        stSquare      : if (w<=1) or (h<=1) then
                         begin
                           MoveTo (x,y);
                           if w<=1 then LineTo (x{+w},y+h)
                                   else LineTo (x+w,y{+h});
                         end
                        else
                           Rectangle (x,y,x+w,y+h);
        stRoundRect,
        stRoundSquare : RoundRect (x,y,x+w,y+h,s div 4, s div 4);
        stEllipse,
        stCircle      : Ellipse (x,y,x+W,y+h);
     end;
   end;
end;

///////////////////////////////////////////////////////////////////////////////

constructor tAlImage.Create (AOwner:tComponent);
begin
  inherited Create (AOwner);
  inherited Autosize := false;
  FStretch  := true;
  FCenter   := true;
  height    := 100;
  width     := 100;

  FPicture  := tPicture.Create;
  aPen := tPen.Create;    { used when no picture is loaded }
  aPen.Color := clBlack;
  aPen.Style := psDashDot;
end;

destructor tAlImage.Destroy;
begin
  aPen.Free;
  FPicture.Free;
  inherited destroy;
end;

procedure tAlImage.WMSize (var Message: TWMSize);
begin
end;

procedure tAlImage.PicChanged (Sender:tObject);
begin
  if csLoading in ComponentState then
     { do not draw image }
  else
   begin
     if Autosize then
      begin
        width  := Picture.Width;
        height := Picture.height;
      end;
     Invalidate;
   end;
end;

procedure tAlImage.Paint;
var
  x,y : integer;
begin
  if FPicture.Graphic is tBitmap then
   begin
     if FStretch then
        Canvas.StretchDraw (Rect(0,0,width,height),FPicture.Graphic)
     else if FCenter then
      begin
        x := (width-FPicture.width) div 2;
        y := (height-FPicture.height) div 2;
        Canvas.Draw (x,y,FPicture.Graphic);
      end
     else
        Canvas.Draw (0,0,FPicture.Graphic);
   end
  else with Canvas do
   begin
     Pen.Assign(aPen);
     Rectangle (0,0,width,height);
   end;
end;

procedure tAlImage.PrintOut (var aRect:tRect; const dummy:string);
var
  ax,ay : integer;
  aBitmap : tBitmap;
begin
  if FPicture.Graphic is tBitmap then
   begin
     if FStretch then
        AlPrinter.Canvas.StretchDraw (aRect,FPicture.Graphic)
     else
      begin
        if FCenter then
         begin
           ax := (Width-FPicture.Width) div 2;
           ay := (Height-FPicture.Height) div 2;
         end
        else
         begin
           ax := 0;
           ay := 0;
         end;
        aBitmap := tBitmap.Create;
        aBitmap.Height := Height;
        aBitmap.Width  := Width;
        aBitmap.Canvas.Draw (ax,ay,FPicture.Graphic);
        AlPrinter.Canvas.StretchDraw (aRect,aBitmap);
        aBitmap.Free;
      end;
   end
  else
   begin
     AlPrinter.Canvas.Pen.Assign(aPen);
     AlPrinter.Canvas.Rectangle (aRect.Left,aRect.Top,
                    aRect.Right-aRect.Left,aRect.Bottom-aRect.Top);
   end;
end;

procedure tAlImage.SetCenter(value:boolean);
begin
  if value<>FCenter then
   begin
     FCenter := value;
     Invalidate;
   end;
end;

procedure tAlImage.SetStretch(value:boolean);
begin
  if value<>FStretch then
   begin
     FStretch := value;
     Invalidate;
   end;
end;

procedure tAlImage.SetAutosize(value:boolean);
begin
  if value<>Autosize then
   begin
     inherited SetAutoSize(value);
     if value then
      begin
        width := Picture.Width;
        height := Picture.height;
      end;
     Invalidate;
   end;
end;

procedure tAlImage.SetPicture(value:tPicture);
begin
  FPicture.Assign(value);
  if Autosize then
   begin
     width := Picture.Width;
     height := Picture.height;
   end;
  Invalidate;
end;

///////////////////////////////////////////////////////////////////////////////

function FindReport (aForm:tForm):tAlReport;
var
  i : integer;
begin
  result := NIL;
  for i:=0 to aForm.ComponentCount-1 do
     if aForm.Components[i] is tAlReport then
        result := aForm.Components[i] as tAlReport;
end;

function LoadReport (const aFileName:string;
                     VAR aReport:tAlReport; VAR aForm:tForm):boolean;
begin
  result := false;
  if FileExists (aFileName) then
   begin
     try
       aForm   := tForm.Create (Application);
       aForm   := ReadComponentResFile (aFileName,aForm) as tForm;
       aReport := FindReport (aForm);
       result := Assigned(aReport);
     except
       on E:Exception do
        begin
          aForm.Free;
          MessageDlg (e.message,mtError,[mbOK],0);
        end;
     end; // try
   end; // if FileExists
end;

procedure Register;
begin
  RegisterComponents('AlRep', [TAlReport,tAlBand,TAlField,TAlLabel,
                               TAlSysField,TAlShape,TAlImage,TAlDetailLink]);
end;

Initialization
  FAlPrinter := NIL;
  NoValue := '~';
  RegisterClasses ([TAlReport,TAlBand,TAlField,TAlLabel,
                    TAlSysField,TAlShape,
                    TAlImage,TAlDetailLink]);
finalization
  if assigned(FAlPrinter) then
     FAlPrinter.Free;
end.
